# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import fields, models


class ResPartner(models.Model):
    _name = "res.partner"
    _inherit = ["portal.mixin", "res.partner"]

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    release_ids = fields.One2many(
        comodel_name="res.partner.release", inverse_name="partner_id", string="Releases"
    )
    module_version_ids = fields.One2many(
        comodel_name="res.partner.module.version",
        inverse_name="partner_id",
        string="Modules",
    )
    release_share_link = fields.Char(
        string="Lien de partage client",
        compute="_compute_release_share_link",
    )
    server_ip = fields.Char("IP du serveur")

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    def _compute_release_share_link(self):
        for partner in self:
            if partner.access_token:
                partner.release_share_link = (
                    partner.get_base_url()
                    + "/releases?partner="
                    + str(partner.id)
                    + "&access_token="
                    + partner.access_token
                )
            else:
                partner.release_share_link = ""

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def set_release_url(self):
        for partner in self:
            if partner.is_company:
                partner._portal_ensure_token()

    def retrieve_modules(self, modules):
        existing_modules = self.env["res.partner.module"].search([])
        ModuleVersion = self.env["res.partner.module.version"]
        for partner in self:
            for module in modules:
                if module.get("author_type") != "odoo":
                    # Création du module
                    if not module.get("name") in existing_modules.mapped("name"):
                        new_module = module
                        new_module.pop("latest_version", False)
                        module_id = existing_modules.create(new_module)
                    # Mise à jour des versions
                    else:
                        module_id = existing_modules.filtered(
                            lambda m: m.name == module.get("name")
                        )
                    module_update = ModuleVersion.search(
                        [
                            ("partner_id", "=", partner.id),
                            ("module_id", "=", module_id.id),
                        ]
                    )
                    if module_update:
                        module_update.update(
                            {"latest_version": module.get("latest_version")}
                        )
                    else:
                        module_update.create(
                            {
                                "partner_id": partner.id,
                                "module_id": module_id.id,
                                "latest_version": module.get("latest_version"),
                            }
                        )
            # Check modules désinstallés
            module_uninstalled = list(
                set(partner.module_version_ids.mapped("module_id.name"))
                - set(
                    map(
                        lambda m: m.get("name")
                        if m.get("author_type") != "odoo"
                        else None,
                        modules,
                    )
                )
            )
            if module_uninstalled:
                module_ids = partner.module_version_ids.filtered(
                    lambda m: m.module_id.name in module_uninstalled
                )
                module_ids.unlink()

    # ------------------------------------------------------
    # Common function
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Portal methods
    # ------------------------------------------------------
