# Copyright 2022 Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)


from odoo import api, fields, models


class PartnerReleaseModule(models.Model):
    _name = "res.partner.release.module"
    _description = "Modules de la release"
    _order = "create_date desc"
    _rec_name = "module_id"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    # Related fields
    release_date = fields.Datetime(
        string="Date release", related="release_id.release_date"
    )
    partner_id = fields.Many2one(string="Client", related="release_id.partner_id")
    # New fields
    release_id = fields.Many2one(
        comodel_name="res.partner.release",
        string="Releases",
        ondelete="cascade",
        copy=False,
    )
    module_id = fields.Many2one(
        comodel_name="res.partner.module",
        string="Module",
        required=True,
        ondelete="restrict",
        copy=False,
    )
    log = fields.Text("Liste des changements")
    release_type = fields.Selection(
        selection=[
            ("fix", "[fix]"),
            ("update", "[update]"),
            ("add", "[add]"),
            ("clean", "[clean]"),
        ],
        string="Type",
    )

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------
    @api.onchange("partner_id")
    def _onchange_partner_id(self):
        module_ids = self.partner_id.module_version_ids.mapped("module_id")
        return {"domain": {"module_id": [("id", "in", module_ids.ids)]}}

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
